'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _getLevel = require('./get-level');

var _getLevel2 = _interopRequireDefault(_getLevel);

var _hasChildren = require('./has-children');

var _hasChildren2 = _interopRequireDefault(_hasChildren);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var toggleChildren = function toggleChildren() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      getRows = _ref.getRows,
      getShowingChildren = _ref.getShowingChildren,
      toggleShowingChildren = _ref.toggleShowingChildren,
      props = _ref.props,
      _ref$idField = _ref.idField,
      idField = _ref$idField === undefined ? 'id' : _ref$idField,
      parentField = _ref.parentField;

  if (!getRows) {
    throw new Error('tree.toggleChildren - Missing getRows!');
  }

  if (!getShowingChildren) {
    throw new Error('tree.toggleChildren - Missing getShowingChildren!');
  }

  if (!toggleShowingChildren) {
    throw new Error('tree.toggleChildren - Missing toggleShowingChildren!');
  }

  var toggle = function toggle(e, index) {
    e.stopPropagation();
    e.preventDefault();

    toggleShowingChildren(index);
  };

  return function (value, extra) {
    var _ref2 = props || {},
        className = _ref2.className,
        restProps = _objectWithoutProperties(_ref2, ['className']); // eslint-disable-line react/prop-types


    var rows = getRows();
    var showingChildren = getShowingChildren(extra);
    // index must be the index of original rows, but filtered rows
    var index = rows.findIndex(function (row) {
      return row[idField] === value;
    });
    var containsChildren = (0, _hasChildren2.default)({ index: index, idField: idField, parentField: parentField })(rows) ? 'has-children' : '';
    var level = (0, _getLevel2.default)({ index: index, idField: idField, parentField: parentField })(rows);
    var hasParent = level > 0 ? 'has-parent' : '';

    return _react2.default.createElement(
      'div',
      _extends({
        style: { paddingLeft: level + 'em' }
        // toggling children with a doubleClick would provide better UX
        // since toggling with a click makes it difficult to select each item.
        , onDoubleClick: function onDoubleClick(e) {
          toggle(e, index);
          window.getSelection && window.getSelection().removeAllRanges();
        },
        className: containsChildren + ' ' + hasParent + ' ' + (className || '')
      }, restProps),
      containsChildren && _react2.default.createElement('span', {
        className: showingChildren ? 'show-less' : 'show-more',
        onClick: function onClick(e) {
          return toggle(e, index);
        }
      }),
      value
    );
  };
};

exports.default = toggleChildren;